#!/usr/bin/env ruby


# #############################################################################
#  ATTENTION
# #############################################################################
#
#  This software is for guidance only. It is provided "as is".
#  It only aims at providing coding information to the customer in order for
#  them to save time. As a result no warranties, whether express, implied or
#  statutory, including, but not limited to implied warranties of
#  merchantability and fitness for a particular purpose apply to this software.
#  A·P·E Angewandte Physik und Elektronik GmbH shall not be held liable for any
#  direct, indirect or consequential damages with respect to any claims arising
#  from the content of such sourcecode and/or the use made by customers of the
#  coding information contained herein in connection with their products.
#
#  File Version: 1.0
#
#  Changelog:
#    1.0 - Initial Version
#
# #############################################################################


#-------------------------------------------------------------------------------
# REQUIRED LIBRARIES / INCLUDES
#-------------------------------------------------------------------------------

#TCPSocket Library, used to open a TCP connection to the device
require 'Socket'

#-------------------------------------------------------------------------------
# GLOBAL VARIABLES
#-------------------------------------------------------------------------------

#Variable to store the  TCPSocket Object
$dev = nil

#-------------------------------------------------------------------------------
# KERNEL TRAPS (CAPTURE FUNCTIONS FOR KEYSTROKES)
#-------------------------------------------------------------------------------

#Capture STRG-C Keystroke to execute an ordered program shutdown instead 
#of a "rude" program termination that may leave the socket connection open
trap("SIGINT") { 
	puts "SIGINT received. Closing TCP Socket and shutting down..."
	$dev.close	#Close TCP-Socket Connection
	$dev = nil	#Set global variable invalid
	exit!		#Terminate Program (exit! is not interruptable)
}


#-------------------------------------------------------------------------------
# GENERAL FUNCTIONS
#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
#Check if the socket conenction is available
#
#Parameters:
#	none
#Return:
#	true, if connection exists
# 	false, if no connection exists
#-------------------------------------------------------------------------------
def connected?
	#Check if the tcp connection exists, return true/false
	return false if !$dev 

	return true
end

#-------------------------------------------------------------------------------
#Connect to a network-ressource
#
#Parameters:
#	host - IP-Address or Hostname of the Connection-Target
#	port - TCP Port Number
#
#Return:
#	true, if connection was established successfully
# 	false, if an error occurred during connection buildup
#-------------------------------------------------------------------------------
def connect host, port

	#Check if connection already exists
	if connected?
		puts "Error. Already connected."
		return false
	end

	begin
		socket = TCPSocket.new( host, port ) 

	#Catch all other errors and display error message
	rescue => e
		puts "Error during connect. Connection could not be established"
		puts e
		$dev = nil #Set global variable invalid
		return false
	end
	
	#Set global variable
	$dev = socket
	
	#Wait 1 second before continuing 
	sleep(0.2)
	
	#If progrm reached this point no error occurred and socket is now connected
	return true
end

#-------------------------------------------------------------------------------
#Close the socket connection and sets the global variable invaid
#
#Parameters:
#	none
#
#Return:
#	none
#-------------------------------------------------------------------------------
def disconnect
	if !connected?
		return false
	end
	
	#Close socket connection
	$dev.close

	#Set global varibale invalid
	$dev = nil
	
	return true
end

#-------------------------------------------------------------------------------
#Send a command via the socket
#
#Parameters:
#	command - Command to send, whitespace characters at the end of the string 
#			  will be replaced with \r\n
#
#Return:
#	Number of characters sent
#-------------------------------------------------------------------------------
def send command
	return $dev.write(command.chomp+"\r\n")
end

#-------------------------------------------------------------------------------
#Receive a specified amount of data
#
#Parameters:
#	length - Number of characters to read from the socket
#
#Return:
#	Received Data as String
#-------------------------------------------------------------------------------
def receive length=1
	return $dev.read(length)
end

#-------------------------------------------------------------------------------
#Send a command and wait/read the returned answer (single line only)
#
#Parameters:
#	command - Command to send, whitespace characters at the end of the string 
#			  will be replaced with \r\n
#
#Return:
#	Received data; Function blocks program-flow until data was received.
#-------------------------------------------------------------------------------
def query command
	$dev.write(command.chomp+"\r\n")
	return $dev.gets
end

#-------------------------------------------------------------------------------
# MAIN PROGRAM
#-------------------------------------------------------------------------------

#Set hostname (or IP) and tcp port for the connection
con_host = "localhost"
#con_host = "127.0.0.1" #Alternative
con_port = 51123	#The portnumber specified in the APE Software

#Connect to specified host and request device information
if connect(con_host, con_port)
	puts "Connected to #{con_host}:#{con_port}"
	puts "Requesting Deviceinformation (*IDN?)..."
	puts "Answer: " + query("*IDN?")+"\n"
	disconnect()
end
	
exit